@@ -78,26 +78,6 @@ use constant DIGEST_MODS => qw(
    Digest::Perl::MD5
 );
 
-# see _parse();
-use constant MAP_KEYS_CHECK => sub {
-   my $tmp = q(
-         <%BUF%> .= exists <%HASH%>->{"<%KEY%>"}
-                  ? (
-                     defined <%HASH%>->{"<%KEY%>"}
-                     ? <%HASH%>->{"<%KEY%>"}
-                     : "[ERROR] Key not defined: <%KEY%>"
-                     )
-                  : "[ERROR] Invalid key: <%KEY%>"
-                  ;
-   );
-   $tmp =~ s/\n//xmsg;
-   $tmp =~ s/\s{2,}/ /xmsg;
-   return $tmp;
-}->();
-
-use constant MAP_KEYS_INIT     => q(<%BUF%> .= <%HASH%>->{"<%KEY%>"} || '';);
-use constant MAP_KEYS_DEFAULT  => q(<%BUF%> .= <%HASH%>->{"<%KEY%>"};);
-
 use constant FRAGMENT_TMP      => <<'TEMPLATE_CONSTANT';
 
 # BEGIN TIDIED FRAGMENT
@@ -206,9 +186,6 @@ BEGIN {
                         FRAGMENT_TMP
                         DISK_CACHE_COMMENT
                         DISK_CACHE_MARKER
-                        MAP_KEYS_CHECK
-                        MAP_KEYS_INIT
-                        MAP_KEYS_DEFAULT
                      )],
       delims    =>   [qw(
                         DELIM_START
